;*******************************************************************************
; MIT License
;
; Copyright (c) 2020 Hugues Johnson
;
; Permission is hereby granted, free of charge, to any person obtaining a copy
; of this software and associated documentation files (the "Software"), to deal
; in the Software without restriction, including without limitation the rights
; to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
; copies of the Software, and to permit persons to whom the Software is
; furnished to do so, subject to the following conditions:
;
; The above copyright notice and this permission notice shall be included in all
; copies or substantial portions of the Software.
;
; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
; IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
; FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
; AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
; LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
; OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
; SOFTWARE.
;*******************************************************************************


;*******************************************************************************
; This borrows code from RetailClerk89
; This makes the code a little larger than it needs to be maybe
; There is very little optimization in the code
;
; Build this using: 
;	vasmm68k_mot -o 4WayDebug.bin -Fbin -spaces 4WayDebug.X68
;
; Visit https://HuguesJohnson.com/ for other random retro gaming stuff
;*******************************************************************************

;*******************************************************************************
; constants
;*******************************************************************************

;-------------------------------------------------------------------------------
; demo-specific constants
;-------------------------------------------------------------------------------
STATE_FLAG_VBLANK=$0	; flag used to track vblank status
ROW_HEIGHT=$800000	; height of a tile row
FONT_VDP=$1200	; address to store fonts in vdp
ETX=$03	; end of text
DIALOG_BASE_TILE=$8090	; base tile for text
;-------------------------------------------------------------------------------
; general MD constants, not all are used in this demo
;-------------------------------------------------------------------------------
; 68k memory map
CTRL_1_DATA=$00A10003
CTRL_2_DATA=$00A10005
CTRL_X_DATA=$00A10007
CTRL_1_CONTROL=$00A10009
CTRL_2_CONTROL=$00A1000B
CTRL_X_CONTROL=$00A1000D
REG_HWVERSION=$00A10001
REG_TMS=$00A14000
PSG_INPUT=$00C00011
RAM_START=$00FF0000
VDP_DATA=$00C00000
VDP_CONTROL=$00C00004
VDP_COUNTER=$00C00008
Z80_ADDRESS_SPACE=$00A10000
Z80_BUS=$00A11100
Z80_RESET=$00A11200
; VDP access modes
VDP_CRAM_READ=$20000000
VDP_CRAM_WRITE=$C0000000
VDP_VRAM_READ=$00000000
VDP_VRAM_WRITE=$40000000
VDP_VSRAM_READ=$10000000
VDP_VSRAM_WRITE=$14000000
; VDP status
VDP_STATUS_FIFO_EMPTY=$0200
VDP_STATUS_FIFO_FULL=$0100
VDP_STATUS_VINT_PENDING=$0080
VDP_STATUS_SPRITE_OVERFLOW=$0040
VDP_STATUS_SPRITE_COLLISION=$0020
VDP_STATUS_ODD_FRAME=$0010
VDP_STATUS_VBLANK=$0008
VDP_STATUS_HBLANK=$0004
VDP_STATUS_DMA=$0002
VDP_STATUS_PAL=$0001
; buttons
BUTTON_UP=%00000001
BUTTON_DOWN=%00000010
BUTTON_LEFT=%00000100
BUTTON_RIGHT=%00001000
BUTTON_B=%00010000
BUTTON_C=%00100000
BUTTON_A=%01000000
BUTTON_START=%10000000
BUTTON_Z=%00000001
BUTTON_Y=%00000010
BUTTON_X=%00000100
BUTTON_MODE=%00001000
; these are based on registers 2,3,4,5,13 in VDP init data
VDP_VRAM_WRITE_A=$40000003	; register 2
VDP_VRAM_WRITE_HSCROLL=$7C000002	; register 13
VDP_VRAM_WRITE_SPRITE=$78000002	; register 5
VDP_VRAM_WRITE_B=$60000003	; register 4
VDP_VRAM_WRITE_WINDOW=$60000002	; register 3
VDP_VRAM_WRITE_VSCROLL=$40000010	; vscroll write
; other constants
WORD_SIZE=$2
LWORD_SIZE=$4
LWORDS_PER_TILE=$8	; how many lwords of data in each tile
VDP_PLANE_PIXELS_X=$0200	; width of VDP plane
VDP_PLANE_PIXELS_Y=$0100	; height of VDP plane
DISPLAY_PIXELS_X=$0140	; width of physical display
DISPLAY_PIXELS_Y=$00E0	; height of physical display

;-------------------------------------------------------------------------------
; memory map
;-------------------------------------------------------------------------------
MEM_START=$FFFF0000
MEM_GAME_STATE=$FFFF0004
MEM_CONTROL_1_HELD=$FFFF1000
MEM_CONTROL_1_PRESSED=$FFFF1002
MEM_CONTROL_2_HELD=$FFFF1004
MEM_CONTROL_2_PRESSED=$FFFF1006
MEM_CONTROL_3_HELD=$FFFF1008
MEM_CONTROL_3_PRESSED=$FFFF100A
MEM_CONTROL_4_HELD=$FFFF100C
MEM_CONTROL_4_PRESSED=$FFFF100E
MEM_DEBUG_1=$FFFF2000
MEM_DEBUG_2=$FFFF3000
MEM_END=$FFFF4000

;*******************************************************************************
; vectors
;*******************************************************************************
	dc.l	$FFFFFE00		; Stack pointer
	dc.l	Start			; Code start
	dc.l	TrapException	; Bus error
	dc.l	TrapException	; Address error
	dc.l	TrapException	; Illegal instruction
	dc.l	TrapException	; Divistion by zero
	dc.l	TrapException	; CHK exception
	dc.l	TrapException	; TRAPV exception
	dc.l	TrapException	; Privilege violation
	dc.l	TrapException	; TRACE exeption
	dc.l	TrapException	; Line 1010 emulator
	dc.l	TrapException	; Line 1111 emulator
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Spurious exception
	dc.l	TrapException	; Interrupt request level 1 (lowest priority)
	dc.l	ExtInt			; Interrupt request level 2 (external interrupt)
	dc.l	TrapException	; Interrupt request level 3
	dc.l	HBlank			; Interrupt request level 4 (VDP interrupt/HBlank)
	dc.l	TrapException	; Interrupt request level 5
	dc.l	VBlank			; Interrupt request level 6 (VDP interrupt/VBlank)
	dc.l	TrapException	; Interrupt request level 7 (highest priority)
	dc.l	TrapException	; TRAP #00 exception
	dc.l	TrapException	; TRAP #01 exception
	dc.l	TrapException	; TRAP #02 exception
	dc.l	TrapException	; TRAP #03 exception
	dc.l	TrapException	; TRAP #04 exception
	dc.l	TrapException	; TRAP #05 exception
	dc.l	TrapException	; TRAP #06 exception
	dc.l	TrapException	; TRAP #07 exception
	dc.l	TrapException	; TRAP #08 exception
	dc.l	TrapException	; TRAP #09 exception
	dc.l	TrapException	; TRAP #10 exception
	dc.l	TrapException	; TRAP #11 exception
	dc.l	TrapException	; TRAP #12 exception
	dc.l	TrapException	; TRAP #13 exception
	dc.l	TrapException	; TRAP #14 exception
	dc.l	TrapException	; TRAP #15 exception
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved
	dc.l	TrapException	; Reserved

;*******************************************************************************
; ROM header
;*******************************************************************************
	dc.b	'SEGA GENESIS    '	; console name
	dc.b	'(C)HUJO '			; copyright
 	dc.b	'2020.JUN'			; date
	dc.b	'Team Player Debug                               ' ; cart name
	dc.b	'Team Player Debug                               ' ; cart name (alt)
	dc.b	'GM 20200630-00'	; program type / serial number / version
	dc.w	$0000				; ROM checksum
	dc.b	'J               '	; I/O device support (unused)
	dc.l	$00000000			; address of ROM start
	dc.l	RomEnd				; address of ROM end
	dc.l	$FFFF0000,$FFFFFFFF	; RAM start/end
	dc.b	'            '		; backup RAM info
	dc.b	'            '		; modem info
	dc.b	'http://huguesjohnson.com/               ' ; comment
	dc.b	'JUE             '	; regions allowed

;*******************************************************************************
; entry point & initialization
;*******************************************************************************
Start:

	move.w	#$2700,sr		; disable interrupts

SecurityCheck:
	move.b	(REG_HWVERSION),d0	; get the hardware version
	andi.b	#$0F,d0				; the last four bits contains the version
	beq.b	SkipSecurity		; skip the next instruction on v1 hardware
	move.l	#'SEGA',(REG_TMS)	; otherwise copy 'SEGA' to TMS register
SkipSecurity:

InitVDP:
	lea	(VDPInitDataStart),a0	; load the address of the VDP init data to a0
	move.w	#(VDPInitDataEnd-VDPInitDataStart)/2,d1	; length of init data to d1
InitVDPDataLoop:
	move.w	(a0)+,(VDP_CONTROL)	; write the init data
	dbf	d1,InitVDPDataLoop		; decrement value of d1 and loop if not 0

ClearCRAM:
	move.l	#VDP_CRAM_WRITE,(VDP_CONTROL); set VDP ctrl to CRAM write
	move.w	#$003f,d1	; 3f = 63 = (64 palettes in CRAM - 1)
ClearCRAMLoop:
	move.w	#$0000,(VDP_DATA)
	dbf	d1,ClearCRAMLoop

; clear VRAM - prevents garbage tiles from showing at boot on clone systems
ClearVRAM:
	move.l	#VDP_VRAM_WRITE,(VDP_CONTROL) 
	move.w	#$7FFF,d1
ClearVRAMLoop:
	move.w	#$0000,(VDP_DATA)
	dbra	d1,ClearVRAMLoop

ClearRAM:
	lea	MEM_START,a0
	move.w	#(MEM_END-MEM_START),d1
ClearRAMLoop:
	and.l	#$00000000,(a0)+
	dbf	d1,ClearRAMLoop

Init4Way:
	move.b	#$40,(CTRL_1_CONTROL)
	nop
	nop
	move.b	#$43,(CTRL_2_CONTROL)
	nop
	nop
	move.b	#$7C,(CTRL_2_DATA)
	nop
	nop
	move.b	#$7F,(CTRL_2_CONTROL)
	nop
	nop
	move.b	#$7C,(CTRL_2_DATA)
	nop
	nop

	move.w	#$2000,sr	; re-enable interrupts

;*******************************************************************************
; main game program
;*******************************************************************************

Main:

	move.w	#$2700,sr	; disable interrupts

	;---------------------------------------------------------------------------
	; load tiles
	;---------------------------------------------------------------------------
	; font tiles
	move.w	#FONT_VDP,d1	; setup call to BuildVRAMWriteAddress
	bsr.w	BuildVRAMWriteAddress	; build the VRAM write address
	move.l	#FontTilesStart,d2	; start address of tileset
	move.l	#FontTilesEnd,d0	; end address of tileset
	sub.l	d2,d0	; subtract the start address to get length
	divu.w	#$0004,d0	; divide by 4 to setup call to LoadTiles
	movea.l	d2,a0	; set address of first tile to load
	bsr.w	LoadTiles	; branch to LoadTiles subroutine

	;---------------------------------------------------------------------------
	; load palettes
	;---------------------------------------------------------------------------
	lea	(Palette0),a0	; point to first palette
	moveq	#$0004,d0	; number of palettes to load
	move.l	#VDP_CRAM_WRITE,d1	; initial write address
	bsr.w	LoadPalettes	; branch to LoadPalettes subroutine

	;---------------------------------------------------------------------------
	; draw the captions
	;---------------------------------------------------------------------------
	lea	LabelController1ButtonsHeld,a0	; start address of text
	move.w	#DIALOG_BASE_TILE,d0	; base pattern
	movea.l	#VDP_VRAM_WRITE_B+$00020000+$00800000,a1	; drawing location
	bsr.w	DrawText
	lea	LabelController1ButtonsPressed,a0	; start address of text
	move.w	#DIALOG_BASE_TILE,d0	; base pattern
	movea.l	#VDP_VRAM_WRITE_B+$00020000+$01000000,a1	; drawing location
	bsr.w	DrawText
	lea	LabelController2ButtonsHeld,a0	; start address of text
	move.w	#DIALOG_BASE_TILE,d0	; base pattern
	movea.l	#VDP_VRAM_WRITE_B+$00020000+$01800000,a1	; drawing location
	bsr.w	DrawText
	lea	LabelController2ButtonsPressed,a0	; start address of text
	move.w	#DIALOG_BASE_TILE,d0	; base pattern
	movea.l	#VDP_VRAM_WRITE_B+$00020000+$02000000,a1	; drawing location
	bsr.w	DrawText
	lea	LabelController3ButtonsHeld,a0	; start address of text
	move.w	#DIALOG_BASE_TILE,d0	; base pattern
	movea.l	#VDP_VRAM_WRITE_B+$00020000+$02800000,a1	; drawing location
	bsr.w	DrawText
	lea	LabelController3ButtonsPressed,a0	; start address of text
	move.w	#DIALOG_BASE_TILE,d0	; base pattern
	movea.l	#VDP_VRAM_WRITE_B+$00020000+$03000000,a1	; drawing location
	bsr.w	DrawText
	lea	LabelController4ButtonsHeld,a0	; start address of text
	move.w	#DIALOG_BASE_TILE,d0	; base pattern
	movea.l	#VDP_VRAM_WRITE_B+$00020000+$03800000,a1	; drawing location
	bsr.w	DrawText
	lea	LabelController4ButtonsPressed,a0	; start address of text
	move.w	#DIALOG_BASE_TILE,d0	; base pattern
	movea.l	#VDP_VRAM_WRITE_B+$00020000+$04000000,a1	; drawing location
	bsr.w	DrawText
	lea	LabelDebug,a0	; start address of text
	move.w	#DIALOG_BASE_TILE+%0010000000000000,d0	; base pattern
	movea.l	#VDP_VRAM_WRITE_B+$00020000+$05000000,a1	; drawing location
	bsr.w	DrawText

	move.w	#$2000,sr		; re-enable interrupts

MainLoop:
	bsr.w	WaitVBlank ; wait for vblank to complete

	move.w	#$2700,sr		; disable interrupts

	;----------------------------
	; write debug values
	;----------------------------
	moveq	#$07,d4	; use d4 for loop control
	move.l	#VDP_VRAM_WRITE_B+$00020000+$05800000,d1
	move.l	#VDP_VRAM_WRITE_B+$00140000+$05800000,d2
	move.l	#VDP_VRAM_WRITE_B+$003C0000+$00800000,d3
	lea	(MEM_DEBUG_1),a2
	lea	(MEM_DEBUG_2),a3
	lea	(MEM_CONTROL_1_HELD),a4
WriteDebugLoop:
	movea.l	d1,a1	; drawing location
	move.b	(a2),d6	; value to write
	bsr.w	DrawBin	; branch to DrawBin
	movea.l	d2,a1	; drawing location
	move.b	(a3),d6	; value to write
	bsr.w	DrawBin	; branch to DrawBin
	movea.l	d3,a1	; drawing location
	move.b	(a4),d6	; value to write
	bsr.w	DrawBin	; branch to DrawBin
	; increment stuff
	adda.l	#$00000002,a2	; increment address a2 is pointing to
	adda.l	#$00000002,a3	; increment address a3 is pointing to
	adda.l	#$00000002,a4	; increment address a4 is pointing to
	add.l	#$00800000,d1	; increment vdp write address (left column)
	add.l	#$00800000,d2	; increment vdp write address (center column)
	add.l	#$00800000,d3	; increment vdp write address (right column)
	dbf	d4,WriteDebugLoop

	move.w	#$2000,sr		; re-enable interrupts

	bra.w	MainLoop

;-------------------------------------------------------------------------------
; WaitVBlank
;-------------------------------------------------------------------------------
WaitVBlank:
	btst	#STATE_FLAG_VBLANK,(MEM_GAME_STATE)	; test vblank flag
	beq.s	WaitVBlank	; loop until flag changes
	bclr	#STATE_FLAG_VBLANK,(MEM_GAME_STATE)	; clear vblank flag
	rts	; exit

;*******************************************************************************
; traps
;*******************************************************************************

ExtInt:
	rte

HBlank:
	rte

VBlank:
	bset	#STATE_FLAG_VBLANK,(MEM_GAME_STATE)	; set vblank flag

VBlankReadJoypads:
	bsr.w	Read4Way	; read controllers

VBlankExit:
	rte

TrapException:
	rte

;*******************************************************************************
; subroutines
;*******************************************************************************
;-------------------------------------------------------------------------------
; LoadPalettes
; loads 1-4 palettes into the VDP
; a0 = address of first palette to load
; d0 = number of palettes to load (1-4)
; d1 = VDP write address
;-------------------------------------------------------------------------------
LoadPalettes:
	move.l	d1,(VDP_CONTROL) ; set up VDP write to CRAM
	mulu	#$10,d0	; multiply number of palettes by 16
	subq	#$01,d0	; subtract 1 since the loop counts to 0
LoadPalettesLoop:
	move.w	(a0)+,(VDP_DATA)	; write the palette data
	dbf	d0,LoadPalettesLoop		; decrement value of d0 and loop if not 0
	rts

;-------------------------------------------------------------------------------
; LoadTiles
; loads tiles into the VDP
; a0 = address of first tile to load
; d0 = number of tiles to load
; d1 = VDP write address
;-------------------------------------------------------------------------------
LoadTiles:
	subq	#$01,d0	; iterate over ((number of tiles to load) - 1)
	move.l	d1,(VDP_CONTROL)	; set address for VDP write
LoadTilesLoop:
	move.l (a0)+,(VDP_DATA)	; move data to VDP & increment address
	dbra d0,LoadTilesLoop	; loop until all data is loaded
	rts

;-------------------------------------------------------------------------------
; DrawTileset
; draws a set of tiles 
; Parameters
; a0 = starting address of tileset
; a1 = initial VRAM write address
; d0 = base pattern (ID+palette+high/low) of first tile in the tileset
; d1 = number of times to repeat the pattern
; other registers used
; a2 = used to reference current VRAM write address
; d2 = number of rows in the tileset
; d3 = number of columns in the tileset 
; d4 = used to store the VDP pattern 
; d5 = loop counter
; d6 = loop counter
;-------------------------------------------------------------------------------
DrawTileset:
	move.w	(a0)+,d2	; store the number of rows
	move.w	(a0)+,d3	; store the number of columns
DrawTilesetOuterLoop:
	movea.l	a0,a2	; reset a2 to starting address
	move.w	d2,d5	; reset d5 to store the number of rows to loop over
DrawTilesetRowLoop:
	move.w	d3,d6	; reset d6 to store the number of columns to loop over
	move.l	a1,(VDP_CONTROL)	; set VDP address
DrawTilesetColumnLoop:
	move.w	(a2)+,d4	; store the next tile index in d4
	add.w	d0,d4	; add base tile ID to tile index
	; draw the tile
	move.w  d4,(VDP_DATA)	; copy the pattern to VPD
	dbf	d6,DrawTilesetColumnLoop	; decrement value of d6 (column) and loop 
	adda.l	#ROW_HEIGHT,a1	; move to the next row
	dbf	d5,DrawTilesetRowLoop	; decrement value of d5 (row) and loop
	dbf	d1,DrawTilesetOuterLoop	; decrement value of d1 (repeat) and loop
	rts

;-------------------------------------------------------------------------------
; BuildVRAMWriteAddress
; builds the VRAM write address based on a relative address
; i.e. - passing $2200 should return $62000000
; d1 = relative VDP write address, final result is also stored in d1
; d0 is modified in this routine
;-------------------------------------------------------------------------------
BuildVRAMWriteAddress:
	; the two highest bits in d1 need to be the lowest bits in the final result
	move.w	d1,d0 ; start by copying d1 to d0	
	lsr.w	#$07,d0	; shift bits 7 to the right
	lsr.w	#$07,d0	; shift bits 7 more to the right
	and.l	#%00000000000000000000000000000011,d0	; clear all but lowest two
	; the two highest bits from d1 are now in d0
	swap	d1	; move the value in d1 to the high word
	and.l	#%00111111111111110000000000000000,d1	; clear all but the magic 14
	add.l	d0,d1	; add the value in d0
	add.l	#VDP_VRAM_WRITE,d1	; add the base VRAM write address
	rts

DrawText:
; if the name of this label is odd, reminder it was copied from RetailClerk89
LoadSceneDrawTextNextCharacter:
	clr	d6	; clear out d6
	move.b	(a0),d6	; copy current character to d6
	cmpi.b	#ETX,d6	; is this the end of the text?
	beq.s	LoadSceneDrawTextLoopEnd	; end of this string, move to next one
	sub.w	#$20,d6	; subtract 32 to get the character index
	; draw the character
	add.w	d0,d6	; add the base tile
	move.l	a1,(VDP_CONTROL)	; set VDP address
	move.w  d6,(VDP_DATA)	; copy the character to VPD
	; increment for next character
	adda.l	#$00000001,a0	; move to next character in the string
	add.l	#$00020000,a1	; move to the next column
	bra.s	LoadSceneDrawTextNextCharacter	; repeat until ETX is reached
LoadSceneDrawTextLoopEnd:
	rts

DrawBin:
	moveq	#7,d7
DrawBinLoop:
	btst	d7,d6
	beq.s	.Zero
	move.w	#DIALOG_BASE_TILE+%0110000000010001,d5	; base pattern	
	bra.s	.Draw
.Zero
	move.w	#DIALOG_BASE_TILE+%0110000000010000,d5	; base pattern	
.Draw
	move.l	a1,(VDP_CONTROL)	; set VDP address
	move.w  d5,(VDP_DATA)	; copy the character to VPD
	; increment for next character
	add.l	#$00020000,a1	; move to the next column
	dbpl	d7,DrawBinLoop
	rts

Read4Way:
	; for debug data
	lea	(MEM_DEBUG_1),a2
	lea	(MEM_DEBUG_2),a3
	; point a0 and a1 to controllers
	lea	(CTRL_1_DATA),a0
	lea	(CTRL_2_DATA),a1
	; read controller 1	
	move.b	#$0C,(a1)
	bsr.w	ReadController
	move.b	(MEM_CONTROL_1_HELD),d1
	move.b	d0,(MEM_CONTROL_1_HELD)
	eor.b	d1,d0
	move.b	d0,(MEM_CONTROL_1_PRESSED)
	; read controller 2	
	move.b	#$1C,(a1)
	bsr.w	ReadController
	move.b	(MEM_CONTROL_2_HELD),d1
	move.b	d0,(MEM_CONTROL_2_HELD)
	eor.b	d1,d0
	move.b	d0,(MEM_CONTROL_2_PRESSED)
	; read controller 3	
	move.b	#$2C,(a1)
	bsr.w	ReadController
	move.b	(MEM_CONTROL_3_HELD),d1
	move.b	d0,(MEM_CONTROL_3_HELD)
	eor.b	d1,d0
	move.b	d0,(MEM_CONTROL_3_PRESSED)
	; read controller 4	
	move.b	#$3C,(a1)
	bsr.w	ReadController
	move.b	(MEM_CONTROL_4_HELD),d1
	move.b	d0,(MEM_CONTROL_4_HELD)
	eor.b	d1,d0
	move.b	d0,(MEM_CONTROL_4_PRESSED)
	rts

ReadController:
	move.b	#$00,(a0)	; set TH low
	nop	; bus synchronization
	nop	; bus synchronization
	move.b	(a0),d0	; d0 has 00SA0000
	move.b	#$40,(a0)	; set TH high
	lsl.b	#$02,d0 ; d0 is now SA000000
	andi.b	#%11000000,d0	; clear lowest 6 bit
	move.b	(a0),d1	; d1 now has 00BCRLDU

;## DEBUG CODE ###############
	move.b	d0,(a2)
	move.b	d1,(a3)
	adda.l	#$00000002,a2
	adda.l	#$00000002,a3
;#############################

	andi.b	#%00111111,d1	; clear 2 highest bits
	or.b	d1,d0	; merge the two controller reads into d0
	; d0 now has SABCRLDU
	not.b	d0 	; flip bit so 0 means not pressed and 1 means pressed

;## DEBUG CODE ###############
	move.b	d0,(a2)
	move.b	d1,(a3)
	adda.l	#$00000002,a2
	adda.l	#$00000002,a3
;#############################

	rts

;*******************************************************************************
; initialization data
;*******************************************************************************
;-------------------------------------------------------------------------------
; Z80 initialization data
;-------------------------------------------------------------------------------
Z80InitDataStart:
	dc.w	$af01
	dc.w	$d91f
	dc.w	$1127
	dc.w	$0021
	dc.w	$2600
	dc.w	$f977 
	dc.w    $edb0
	dc.w	$dde1
	dc.w	$fde1
	dc.w	$ed47
	dc.w	$ed4f
	dc.w	$d1e1                                   
	dc.w    $f108
	dc.w	$d9c1
	dc.w	$d1e1
	dc.w	$f1f9
	dc.w	$f3ed
	dc.w	$5636
	dc.w	$e9e9 
Z80InitDataEnd:

;-------------------------------------------------------------------------------
; VDP initialization data
;-------------------------------------------------------------------------------
VDPInitDataStart:
	dc.w	$8004
	dc.w	$8174
	dc.w	$8230
	dc.w	$8328
	dc.w	$8407
	dc.w	$855C
	dc.w	$8600
	dc.w	$8700
	dc.w	$8800
	dc.w	$8900
	dc.w	$8A00
	dc.w	$8B00
	dc.w	$8C81
	dc.w	$8D2F
	dc.w	$8E00
	dc.w	$8F02
	dc.w	$9001
	dc.w	$9100
	dc.w	$9200
	dc.w	$93FF
	dc.w	$94FF
	dc.w	$9500
	dc.w	$9600
	dc.w	$9780
VDPInitDataEnd:

;*******************************************************************************
; palettes
;*******************************************************************************
Palette0:
	;		 0000BBBSGGGSRRRS
	dc.w	%0000000000000000	; 00
	dc.w	%0000000000000000	; 01
	dc.w	%0000000000000000	; 02
	dc.w	%0000101011101010	; 03
	dc.w	%0000000000000000	; 04
	dc.w	%0000000000000000	; 05
	dc.w	%0000000000000000	; 06
	dc.w	%0000000000000000	; 07
	dc.w	%0000000000000000	; 08
	dc.w	%0000000000000000	; 09
	dc.w	%0000000000000000	; 0A
	dc.w	%0000000000000000	; 0B
	dc.w	%0000000000000000	; 0C
	dc.w	%0000000000000000	; 0D
	dc.w	%0000000000000000	; 0E
	dc.w	%0000000000000000	; 0F

Palette1:
	;		 0000BBBSGGGSRRRS
	dc.w	%0000000000000000	; 00
	dc.w	%0000000000000000	; 01
	dc.w	%0000000000000000	; 02
	dc.w	%0000111001001110	; 03
	dc.w	%0000000000000000	; 04
	dc.w	%0000000000000000	; 05
	dc.w	%0000000000000000	; 06
	dc.w	%0000000000000000	; 07
	dc.w	%0000000000000000	; 08
	dc.w	%0000000000000000	; 09
	dc.w	%0000000000000000	; 0A
	dc.w	%0000000000000000	; 0B
	dc.w	%0000000000000000	; 0C
	dc.w	%0000000000000000	; 0D
	dc.w	%0000000000000000	; 0E
	dc.w	%0000000000000000	; 0F

Palette2:
	;		 0000BBBSGGGSRRRS
	dc.w	%0000000000000000	; 00
	dc.w	%0000000000000000	; 01
	dc.w	%0000000000000000	; 02
	dc.w	%0000101011101000	; 03
	dc.w	%0000000000000000	; 04
	dc.w	%0000000000000000	; 05
	dc.w	%0000000000000000	; 06
	dc.w	%0000000000000000	; 07
	dc.w	%0000000000000000	; 08
	dc.w	%0000000000000000	; 09
	dc.w	%0000000000000000	; 0A
	dc.w	%0000000000000000	; 0B
	dc.w	%0000000000000000	; 0C
	dc.w	%0000000000000000	; 0D
	dc.w	%0000000000000000	; 0E
	dc.w	%0000000000000000	; 0F

Palette3:
	;		 0000BBBSGGGSRRRS
	dc.w	%0000000000000000	; 00
	dc.w	%0000000000000000	; 01
	dc.w	%0000000000000000	; 02
	dc.w	%0000111011101110	; 03
	dc.w	%0000000000000000	; 04
	dc.w	%0000000000000000	; 05
	dc.w	%0000000000000000	; 06
	dc.w	%0000000000000000	; 07
	dc.w	%0000000000000000	; 08
	dc.w	%0000000000000000	; 09
	dc.w	%0000000000000000	; 0A
	dc.w	%0000000000000000	; 0B
	dc.w	%0000000000000000	; 0C
	dc.w	%0000000000000000	; 0D
	dc.w	%0000000000000000	; 0E
	dc.w	%0000000000000000	; 0F

;*******************************************************************************
; tiles
;*******************************************************************************
FontTilesStart:

;Font is based off Deathwake font by Zee-3
; http://www.zee-3.com/pickfordbros/archive/bitmapfonts.php
; "Feel free to use any of these fonts for whatever purpose you see fit."


	; (blank)	
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	
	; !
	dc.l	$00000000
	dc.l	$00300300
	dc.l	$00300300
	dc.l	$00300300
	dc.l	$00300300
	dc.l	$00300300
	dc.l	$00000000
	dc.l	$00300300

	; "
	dc.l	$00303000
	dc.l	$00303000
	dc.l	$00303000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000

	; #
	dc.l	$00000000
	dc.l	$00030003
	dc.l	$00300030
	dc.l	$03333330
	dc.l	$00300300
	dc.l	$03333330
	dc.l	$00300300
	dc.l	$03003000

	; $
	dc.l	$00003000
	dc.l	$00033330
	dc.l	$00303000
	dc.l	$00033300
	dc.l	$00003030
	dc.l	$00303030
	dc.l	$00033300
	dc.l	$00003000

	; %
	dc.l	$00000000
	dc.l	$00300003
	dc.l	$03030030
	dc.l	$00300300
	dc.l	$00003000
	dc.l	$00030030
	dc.l	$00300303
	dc.l	$03000030

	; &
	dc.l	$00000000
	dc.l	$00330000
	dc.l	$03003000
	dc.l	$00330000
	dc.l	$00303000
	dc.l	$03000303
	dc.l	$03000330
	dc.l	$00333003

	; '
	dc.l	$00000000
	dc.l	$00033000
	dc.l	$00033000
	dc.l	$00003000
	dc.l	$00030000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000

	; (
	dc.l	$00000000
	dc.l	$00000003
	dc.l	$00000030
	dc.l	$00000300
	dc.l	$00000300
	dc.l	$00000300
	dc.l	$00000030
	dc.l	$00000003

	; )
	dc.l	$00000000
	dc.l	$00000300
	dc.l	$00000030
	dc.l	$00000003
	dc.l	$00000003
	dc.l	$00000003
	dc.l	$00000030
	dc.l	$00000300

	; *
	dc.l	$00000000
	dc.l	$03003003
	dc.l	$00303030
	dc.l	$00033300
	dc.l	$03333333
	dc.l	$00033300
	dc.l	$00303030
	dc.l	$03003003

	; +
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00333330
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00000000

	; ,
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00030000

	; -
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$03333330
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000

	; .
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00003000
	dc.l	$00000000

	; /
	dc.l	$00000003
	dc.l	$00000030
	dc.l	$00000300
	dc.l	$00003000
	dc.l	$00030000
	dc.l	$00300000
	dc.l	$03000000
	dc.l	$30000000

	; 0
	dc.l	$00000000
	dc.l	$00033300
	dc.l	$00300330
	dc.l	$00303030
	dc.l	$00330030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00033300

	; 1
	dc.l	$00000000
	dc.l	$00003000
	dc.l	$00033000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00333330

	; 2
	dc.l	$00000000
	dc.l	$00033300
	dc.l	$00000030
	dc.l	$00033300
	dc.l	$00300000
	dc.l	$00300000
	dc.l	$00300000
	dc.l	$00333330

	; 3
	dc.l	$00000000
	dc.l	$00333330
	dc.l	$00000300
	dc.l	$00003300
	dc.l	$00000030
	dc.l	$00000030
	dc.l	$00300030
	dc.l	$00033300

	; 4
	dc.l	$00000000
	dc.l	$00300000
	dc.l	$00303000
	dc.l	$00333330
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000

	; 5
	dc.l	$00000000
	dc.l	$00333330
	dc.l	$00300000
	dc.l	$00333300
	dc.l	$00000030
	dc.l	$00000030
	dc.l	$00300030
	dc.l	$00033300

	; 6
	dc.l	$00000000
	dc.l	$00033300
	dc.l	$00300000
	dc.l	$00333300
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00033300

	; 7
	dc.l	$00000000
	dc.l	$00333330
	dc.l	$00000030
	dc.l	$00000300
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000

	; 8
	dc.l	$00000000
	dc.l	$00033300
	dc.l	$00300030
	dc.l	$00033300
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00033300

	; 9
	dc.l	$00000000
	dc.l	$00033330
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00033330
	dc.l	$00000030
	dc.l	$00000030
	dc.l	$00000030

	; :
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00030000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00030000
	dc.l	$00000000

	; ;
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00003000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00003000
	dc.l	$00030000
	dc.l	$00000000

	; <
	dc.l	$00000000
	dc.l	$00000330
	dc.l	$00033000
	dc.l	$03300000
	dc.l	$30000000
	dc.l	$03300000
	dc.l	$00033000
	dc.l	$00000330

	; =
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$03333330
	dc.l	$00000000
	dc.l	$03333330
	dc.l	$00000000
	dc.l	$00000000

	; >
	dc.l	$00000000
	dc.l	$03300000
	dc.l	$00033000
	dc.l	$00000330
	dc.l	$00000003
	dc.l	$00000330
	dc.l	$00033000
	dc.l	$03300000

	; ?
	dc.l	$00000000
	dc.l	$00333300
	dc.l	$00000030
	dc.l	$00003300
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00000000
	dc.l	$00003000

	; @
	dc.l	$00000000
	dc.l	$00033300
	dc.l	$00300030
	dc.l	$03033303
	dc.l	$03030303
	dc.l	$03003330
	dc.l	$00300000
	dc.l	$00033300

	; A
	dc.l	$00000000
	dc.l	$00033300
	dc.l	$00300030
	dc.l	$00333330
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030

	; B
	dc.l	$00000000
	dc.l	$00333300
	dc.l	$00300030
	dc.l	$00333300
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00333300

	; C
	dc.l	$00000000
	dc.l	$00033300
	dc.l	$00300030
	dc.l	$00300000
	dc.l	$00300000
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00033300

	; D
	dc.l	$00000000
	dc.l	$00333300
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300300
	dc.l	$00333000

	; E
	dc.l	$00000000
	dc.l	$00333330
	dc.l	$00300000
	dc.l	$00333330
	dc.l	$00300000
	dc.l	$00300000
	dc.l	$00300000
	dc.l	$00333330

	; F
	dc.l	$00000000
	dc.l	$00333330
	dc.l	$00300000
	dc.l	$00333330
	dc.l	$00300000
	dc.l	$00300000
	dc.l	$00300000
	dc.l	$00300000

	; G
	dc.l	$00000000
	dc.l	$00333000
	dc.l	$03000000
	dc.l	$03003330
	dc.l	$03000030
	dc.l	$03000030
	dc.l	$03000030
	dc.l	$00333300

	; H
	dc.l	$00000000
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00333330
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030

	; I
	dc.l	$00000000
	dc.l	$03333300
	dc.l	$00030000
	dc.l	$00030000
	dc.l	$00030000
	dc.l	$00030000
	dc.l	$00030000
	dc.l	$03333300

	; J
	dc.l	$00000000
	dc.l	$00003300
	dc.l	$00000300
	dc.l	$00000300
	dc.l	$00000300
	dc.l	$03000300
	dc.l	$03000300
	dc.l	$00333000

	; K
	dc.l	$00000000
	dc.l	$03003000
	dc.l	$03030000
	dc.l	$03333000
	dc.l	$03000300
	dc.l	$03000300
	dc.l	$03000300
	dc.l	$03000300

	; L
	dc.l	$00000000
	dc.l	$03000000
	dc.l	$03000000
	dc.l	$03000000
	dc.l	$03000000
	dc.l	$03000000
	dc.l	$03000000
	dc.l	$03333300

	; M
	dc.l	$00000000
	dc.l	$00300030
	dc.l	$00330330
	dc.l	$00303030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030

	; N
	dc.l	$00000000
	dc.l	$00300030
	dc.l	$00330030
	dc.l	$00303030
	dc.l	$00300330
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030

	; O
	dc.l	$00000000
	dc.l	$00033300
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00033300

	; P
	dc.l	$00000000
	dc.l	$00333300
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00333300
	dc.l	$00300000
	dc.l	$00300000
	dc.l	$00300000

	; Q
	dc.l	$00000000
	dc.l	$00033300
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00303030
	dc.l	$00300330
	dc.l	$00033330

	; R
	dc.l	$00000000
	dc.l	$00333300
	dc.l	$00300030
	dc.l	$00333300
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030

	; S
	dc.l	$00000000
	dc.l	$00033300
	dc.l	$00300000
	dc.l	$00033300
	dc.l	$00000030
	dc.l	$00000030
	dc.l	$00300030
	dc.l	$00033300

	; T
	dc.l	$00000000
	dc.l	$00333330
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000

	; U
	dc.l	$00000000
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00033300

	; V
	dc.l	$00000000
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00030300
	dc.l	$00003000

	; W
	dc.l	$00000000
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00303030
	dc.l	$00330330
	dc.l	$00300030

	; X
	dc.l	$00000000
	dc.l	$00300030
	dc.l	$00030300
	dc.l	$00003000
	dc.l	$00030300
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030

	; Y
	dc.l	$00000000
	dc.l	$03000300
	dc.l	$00303000
	dc.l	$00030000
	dc.l	$00030000
	dc.l	$00030000
	dc.l	$00030000
	dc.l	$00030000

	; Z
	dc.l	$00000000
	dc.l	$00333330
	dc.l	$00000300
	dc.l	$00003000
	dc.l	$00030000
	dc.l	$00300000
	dc.l	$00300000
	dc.l	$00333330

	; [
	dc.l	$00000000
	dc.l	$00003300
	dc.l	$00030000
	dc.l	$00030000
	dc.l	$00030000
	dc.l	$00030000
	dc.l	$00030000
	dc.l	$00003300

	; \
	dc.l	$00000000
	dc.l	$03000000
	dc.l	$00300000
	dc.l	$00030000
	dc.l	$00003000
	dc.l	$00000300
	dc.l	$00000030
	dc.l	$00000003

	; ]
	dc.l	$00000000
	dc.l	$00330000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00330000

	; ^
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00033333
	dc.l	$00003330
	dc.l	$00000300

	; _
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$03333333

	; `
	dc.l	$00000000
	dc.l	$00033000
	dc.l	$00033000
	dc.l	$00030000
	dc.l	$00003000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000

	; a
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00333000
	dc.l	$00000300
	dc.l	$00333300
	dc.l	$03000300
	dc.l	$03000300
	dc.l	$00333300

	; b
	dc.l	$00000000
	dc.l	$00300000
	dc.l	$00300000
	dc.l	$00303300
	dc.l	$00330030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00333300

	; c
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00033300
	dc.l	$00300030
	dc.l	$00300000
	dc.l	$00300000
	dc.l	$00300030
	dc.l	$00033300

	; d
	dc.l	$00000000
	dc.l	$00000300
	dc.l	$00000300
	dc.l	$00330300
	dc.l	$03003300
	dc.l	$03000300
	dc.l	$03000300
	dc.l	$00333300

	; e
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00333000
	dc.l	$03000300
	dc.l	$03333300
	dc.l	$03000000
	dc.l	$03000000
	dc.l	$00333000

	; f
	dc.l	$00000000
	dc.l	$00003300
	dc.l	$00030000
	dc.l	$00033330
	dc.l	$00030000
	dc.l	$00030000
	dc.l	$00030000
	dc.l	$00030000

	; g
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00333300
	dc.l	$03000300
	dc.l	$03000300
	dc.l	$00333300
	dc.l	$00000300
	dc.l	$00333000

	; h
	dc.l	$00000000
	dc.l	$00300000
	dc.l	$00300000
	dc.l	$00303300
	dc.l	$00330030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030

	; i
	dc.l	$00000000
	dc.l	$00003000
	dc.l	$00000000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000

	; j
	dc.l	$00000000
	dc.l	$00000300
	dc.l	$00000000
	dc.l	$00000300
	dc.l	$00000300
	dc.l	$00000300
	dc.l	$00300300
	dc.l	$00033000

	; k
	dc.l	$00000000
	dc.l	$00300000
	dc.l	$00300300
	dc.l	$00303000
	dc.l	$00333300
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030

	; l
	dc.l	$00000000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000

	; m
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00330300
	dc.l	$00303030
	dc.l	$00303030
	dc.l	$00303030
	dc.l	$00303030
	dc.l	$00303030

	; n
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00303300
	dc.l	$00330030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030

	; o
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00033300
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00033300

	; p
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00303300
	dc.l	$00330030
	dc.l	$00300030
	dc.l	$00333300
	dc.l	$00300000
	dc.l	$00300000

	; q
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00330300
	dc.l	$03003300
	dc.l	$03000300
	dc.l	$00333300
	dc.l	$00000300
	dc.l	$00000300

	; r
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00300330
	dc.l	$00303000
	dc.l	$00330000
	dc.l	$00300000
	dc.l	$00300000
	dc.l	$00300000

	; s
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00333000
	dc.l	$03000000
	dc.l	$00333000
	dc.l	$00000300
	dc.l	$03000300
	dc.l	$00333000

	; t
	dc.l	$00000000
	dc.l	$00030000
	dc.l	$00033330
	dc.l	$00030000
	dc.l	$00030000
	dc.l	$00030000
	dc.l	$00030030
	dc.l	$00003300

	; u
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$03000300
	dc.l	$03000300
	dc.l	$03000300
	dc.l	$03000300
	dc.l	$03000300
	dc.l	$00333300

	; v
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00300030
	dc.l	$00030300
	dc.l	$00003000

	; w
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00303030
	dc.l	$00303030
	dc.l	$00303030
	dc.l	$00303030
	dc.l	$00303030
	dc.l	$00030300

	; x
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$03000300
	dc.l	$00303000
	dc.l	$00030000
	dc.l	$00303000
	dc.l	$03000300
	dc.l	$03000300

	; y
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$03000300
	dc.l	$03000300
	dc.l	$03000300
	dc.l	$00333300
	dc.l	$00000300
	dc.l	$00333000

	; z
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$03333330
	dc.l	$00000300
	dc.l	$00003000
	dc.l	$00030000
	dc.l	$00300000
	dc.l	$03333330

	; {
	dc.l	$00000000
	dc.l	$03333300
	dc.l	$30000030
	dc.l	$30000030
	dc.l	$30000030
	dc.l	$30000030
	dc.l	$30000030
	dc.l	$03333300

	; |
	dc.l	$00000000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000
	dc.l	$00003000

	; }
	dc.l	$00000000
	dc.l	$03333300
	dc.l	$32222230
	dc.l	$32002230
	dc.l	$32022230
	dc.l	$32222230
	dc.l	$32222230
	dc.l	$03333300

	; ~
	dc.l	$00000000
	dc.l	$00330030
	dc.l	$03003300
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000
	dc.l	$00000000

FontTilesEnd:

;*******************************************************************************
; strings
;*******************************************************************************
	align 2

LabelController1ButtonsHeld:
	dc.b	"   Controller 1 Buttons Held=",ETX

LabelController1ButtonsPressed:
	dc.b	"Controller 1 Buttons Pressed=",ETX

LabelController2ButtonsHeld:
	dc.b	"   Controller 2 Buttons Held=",ETX

LabelController2ButtonsPressed:
	dc.b	"Controller 2 Buttons Pressed=",ETX

LabelController3ButtonsHeld:
	dc.b	"   Controller 3 Buttons Held=",ETX

LabelController3ButtonsPressed:
	dc.b	"Controller 3 Buttons Pressed=",ETX

LabelController4ButtonsHeld:
	dc.b	"   Controller 4 Buttons Held=",ETX

LabelController4ButtonsPressed:
	dc.b	"Controller 4 Buttons Pressed=",ETX

LabelDebug:
	dc.b	"Debug Data",ETX

;*******************************************************************************
; end of rom
;*******************************************************************************

RomEnd:
	dc.b	%11111111
    end 0
